@@ -7,6 +7,8 @@
     :license: MIT, see LICENSE for more details.
 """
 
+import os
+import os.path
 import time
 import copy
 import operator
@@ -30,24 +32,35 @@ native_types = [ "int", "float", "string", "bool" ]
 letter       = Range("AZaz")
 digit        = Range("09")
 under        = Str("_")
-ident        = (letter | under) + Rep(letter | digit | under)
+period       = Str(".")
+plain_ident  = (letter | under) + Rep(letter | digit | under)
+ns_ident     = plain_ident + period + plain_ident
+ident        = plain_ident | ns_ident
 arr_ident    = Str("[]") + ident
 space        = Any(" \t\n\r")
+space_tab    = Any(" \t")
 comment      = Str("// ") | Str("//")
 type_opts    = Str("[") + Rep(AnyBut("{}]\n")) + Str("]")
+namespace    = Str("namespace") + Rep(space_tab) + plain_ident
+import_stmt  = Str("import") + Rep(space_tab) + Str('"') + Rep(AnyBut("\"\r\n")) + Str('"')
 
-def parse(idl_text, name=None, validate=True, add_meta=True):
+def file_paths(fname, search_path=None):
+    if not search_path and os.environ.has_key("BARRISTER_PATH"):
+        search_path = os.environ["BARRISTER_PATH"]
+    paths = []
+    paths.append(fname)
+    if search_path:
+        for directory in search_path.split(os.pathsep):
+            paths.append(os.path.join(directory, fname))
+    return paths
+
+def parse(idl_text, idlFilename=None, validate=True, add_meta=True):
     if not isinstance(idl_text, (str, unicode)):
         idl_text = idl_text.read()
 
-    scanner = IdlScanner(idl_text, name)
-    scanner.parse()
+    scanner = IdlScanner(idl_text, idlFilename)
+    scanner.parse(validate=validate)
     
-    if validate:
-        scanner2 = IdlScanner(idl_text, name)
-        scanner2.parse(scanner)
-        scanner = scanner2
-        
     if len(scanner.errors) == 0:
         if add_meta:
             scanner.add_meta()
@@ -55,6 +68,43 @@ def parse(idl_text, name=None, validate=True, add_meta=True):
     else:
         raise IdlParseException(scanner.errors)
 
+def validate_scanner(scanner):
+    scanner2 = IdlScanner(idl_text, idlFilename)
+    scanner2.parse(scanner)
+    scanner = scanner2
+
+def elem_checksum(elem):
+    if elem["type"] == "struct":
+        s = ""
+        fields = copy.copy(elem["fields"])
+        fields.sort(key=operator.itemgetter("name"))
+        for f in fields:
+            fs = (f["name"], f["type"], f["is_array"], f["optional"])
+            s += "\t%s\t%s\t%s\t%s" % fs
+        fs = (elem["name"], elem["extends"], s)
+        return "struct\t%s\t%s\t%s\n" % fs
+    elif elem["type"] == "enum":
+        s = "enum\t%s" % elem["name"]
+        vals = copy.copy(elem["values"])
+        vals.sort(key=operator.itemgetter("value"))
+        for v in vals: s += "\t%s" % v["value"]
+        s += "\n"
+        return s
+    elif elem["type"] == "interface":
+        s = "interface\t%s" % elem["name"]
+        funcs = copy.copy(elem["functions"])
+        funcs.sort(key=operator.itemgetter("name"))
+        for f in funcs:
+            s += "[%s" % f["name"]
+            for p in f["params"]:
+                s += "\t%s\t%s" % (p["type"], p["is_array"])
+            ret = f["returns"]
+            fs = (ret["type"], ret["is_array"], ret["optional"])
+            s += "(%s\t%s\t%s)]" % fs
+        s += "\n"
+        return s
+    return None
+
 class IdlParseException(Exception):
 
     def __init__(self, errors):
@@ -77,10 +127,20 @@ class IdlScanner(Scanner):
         self.parsed = [ ]
         self.errors = [ ]
         self.types = { }
+        self.imports = { }
         self.comment = None
         self.cur = None
+        self.namespace = None
+        self.searchPath = None
+        self.idl_text = idl_text
+        self.name = name
+        if name:
+            searchPath = os.path.dirname(os.path.abspath(name))
+            if os.environ.has_key('BARRISTER_PATH'):
+                searchPath = searchPath + os.pathsep + os.environ['BARRISTER_PATH']
+            self.searchPath = searchPath
 
-    def parse(self, firstPass=None):
+    def parse(self, firstPass=None, validate=False):
         self.firstPass = firstPass
         while True:
             (t, name) = self.read()
@@ -90,6 +150,41 @@ class IdlScanner(Scanner):
                 self.add_error(t)
                 break
 
+        if validate:
+            scanner2 = IdlScanner(self.idl_text, self.name)
+            scanner2.parse(self)
+            self.parsed = scanner2.parsed
+            self.errors = scanner2.errors
+            self.types  = scanner2.types
+
+    def import_file(self, fname):
+        path_to_load = None
+        for path in file_paths(fname, self.searchPath):
+            path = os.path.abspath(path)
+            if os.path.exists(path):
+                path_to_load = path
+                break
+        if path_to_load:
+            if not self.imports.has_key(path_to_load):
+                f = open(path_to_load)
+                idl_text = f.read()
+                f.close()
+                scanner = IdlScanner(idl_text, path_to_load)
+                self.imports[path_to_load] = scanner
+                scanner.parse(validate=True)
+                for elem in scanner.parsed:
+                    if elem["type"] == "struct" or elem["type"] == "enum":
+                        if self.types.has_key(elem["name"]):
+                            c1 = elem_checksum(self.types[elem["name"]])
+                            c2 = elem_checksum(elem)
+                            if c1 != c2:
+                                self.add_error("Include %s redefined type: %s" % (path_to_load, elem["name"]))
+                        else:
+                            self.types[elem["name"]] = elem
+                            self.parsed.append(elem)
+        else:
+            self.add_error("Cannot find import file: %s" % fname)
+
     def eof(self):
         if self.cur:
             self.add_error("Unexpected end of file")
@@ -112,34 +207,8 @@ class IdlScanner(Scanner):
         """
         arr = [ ]
         for elem in self.parsed:
-            if elem["type"] == "struct":
-                s = ""
-                fields = copy.copy(elem["fields"])
-                fields.sort(key=operator.itemgetter("name"))
-                for f in fields:
-                    fs = (f["name"], f["type"], f["is_array"], f["optional"])
-                    s += "\t%s\t%s\t%s\t%s" % fs
-                fs = (elem["name"], elem["extends"], s)
-                arr.append("struct\t%s\t%s\t%s\n" % fs)
-            elif elem["type"] == "enum":
-                s = "enum\t%s" % elem["name"]
-                vals = copy.copy(elem["values"])
-                vals.sort(key=operator.itemgetter("value"))
-                for v in vals: s += "\t%s" % v["value"]
-                s += "\n"
-                arr.append(s)
-            elif elem["type"] == "interface":
-                s = "interface\t%s" % elem["name"]
-                funcs = copy.copy(elem["functions"])
-                funcs.sort(key=operator.itemgetter("name"))
-                for f in funcs:
-                    s += "[%s" % f["name"]
-                    for p in f["params"]:
-                        s += "\t%s\t%s" % (p["type"], p["is_array"])
-                    ret = f["returns"]
-                    fs = (ret["type"], ret["is_array"], ret["optional"])
-                    s += "(%s\t%s\t%s)]" % fs
-                s += "\n"
+            s = elem_checksum(elem)
+            if s:
                 arr.append(s)
         arr.sort()
         #print arr
@@ -249,18 +318,25 @@ class IdlScanner(Scanner):
         if line < 0:
             (name, line, col) = self.position()
         self.errors.append({"line": line, "message": message})
+
+    def prefix_namespace(self, ident):
+        if self.namespace and ident.find(".") < 0 and ident not in native_types:
+            return self.namespace + "." + ident
+        return ident
         
     #####################################################
 
     def begin_struct(self, text):
         self.check_dupe_name(text)
-        self.cur = { "name" : text, "type" : "struct", "extends" : "",
+        name = self.prefix_namespace(text)
+        self.cur = { "name" : name, "type" : "struct", "extends" : "",
                      "comment" : self.get_comment(), "fields" : [] }
         self.begin('start-block')
 
     def begin_enum(self, text):
         self.check_dupe_name(text)
-        self.cur = { "name" : text, "type" : "enum", 
+        name = self.prefix_namespace(text)
+        self.cur = { "name" : name, "type" : "enum", 
                      "comment" : self.get_comment(), "values" : [] }
         self.begin('start-block')
 
@@ -281,6 +357,26 @@ class IdlScanner(Scanner):
             return False
         return True
 
+    def set_namespace(self, text):
+        if self.namespace:
+            self.add_error("Cannot redeclare namespace")
+        elif len(self.parsed) > 0:
+            self.add_error("namespace must preceed all struct/enum/interface definitions")
+        ns = text.strip()[9:].strip()
+        self.namespace = ns
+        self.begin('end_of_line')
+
+    def add_import(self, text):
+        start = text.find('"') + 1
+        end   = text[start:].find('"') + start
+        fname = text[start:end]
+        self.import_file(fname)
+        self.begin('end_of_line')
+
+    def end_of_line(self, text):
+        self.cur = None
+        self.begin('')
+
     def start_block(self, text):
         t = self.cur["type"]
         if t == "struct":
@@ -288,6 +384,8 @@ class IdlScanner(Scanner):
         elif t == "enum":
             self.begin("values")
         elif t == "interface":
+            if self.namespace:
+                self.add_error("namespace cannot be used in files with interfaces")
             self.begin("functions")
         else:
             raise Exception("Invalid type: %s" % t)
@@ -320,8 +418,9 @@ class IdlScanner(Scanner):
         if text.find("[]") == 0:
             text = text[2:]
             is_array = True
-        self.validate_type_vs_first_pass(text)
-        self.field["type"] = text
+        type_name = self.prefix_namespace(text)
+        self.validate_type_vs_first_pass(type_name)
+        self.field["type"] = type_name
         self.field["is_array"] = is_array
         self.field["comment"] = self.get_comment()
         self.field["optional"] = False
@@ -348,8 +447,9 @@ class IdlScanner(Scanner):
         if text.find("[]") == 0:
             text = text[2:]
             is_array = True
-        self.validate_type_vs_first_pass(text)
-        self.param["type"] = text
+        type_name = self.prefix_namespace(text)
+        self.validate_type_vs_first_pass(type_name)
+        self.param["type"] = type_name
         self.param["is_array"] = is_array
         self.function["params"].append(self.param)
         self.param = None
@@ -360,9 +460,10 @@ class IdlScanner(Scanner):
         if text.find("[]") == 0:
             text = text[2:]
             is_array = True
-        self.validate_type_vs_first_pass(text)
+        type_name = self.prefix_namespace(text) 
+        self.validate_type_vs_first_pass(type_name)
         self.function["returns"] = { 
-                "type" : text, 
+                "type" : type_name,
             "is_array" : is_array, 
             "optional" : False }
         self.type = self.function["returns"]
@@ -421,7 +522,7 @@ class IdlScanner(Scanner):
 
     def end_extends(self, text):
         if self.cur and self.cur["type"] == "struct":
-            self.cur["extends"] = text
+            self.cur["extends"] = self.prefix_namespace(text)
             self.validate_struct_extends(self.cur)
         else:
             self.add_error("extends is only supported for struct types")
@@ -434,10 +535,17 @@ class IdlScanner(Scanner):
     lex = Lexicon([
             (Str("\n"),  add_comment_block),
             (space,      IGNORE),
+            (namespace,   set_namespace),
+            (import_stmt, add_import),
             (Str('struct '),   Begin('struct-start')),
             (Str('enum '),   Begin('enum-start')),
             (Str('interface '),   Begin('interface-start')),
             (comment,    start_comment),
+            State('end_of_line', [
+                    (Str("\r\n"), end_of_line),
+                    (Str("\n"), end_of_line),
+                    (space, IGNORE),
+                    (AnyChar, "Illegal character - expected end of line") ]),
             State('struct-start', [
                     (ident,    begin_struct),
                     (space,    IGNORE),
