@@ -11,9 +11,12 @@
     :license: MIT, see LICENSE for more details.
 """
 
+import os
+import os.path
+import inspect
 import time
 import unittest
-from barrister.parser import parse, IdlParseException
+from barrister.parser import parse, file_paths, IdlParseException
 
 def ret_field(type_, is_array=False, optional=False):
     return { "type": type_, "is_array": is_array, "optional": optional }
@@ -421,6 +424,102 @@ interface FooService {
                            "returns" : ret_field("string", optional=True) } ] } ]
         self.assertEquals(expected, parse(idl, add_meta=False))
 
+    def test_prefix_namespace_if_present(self):
+      idl = """
+      namespace common
+
+    struct A {
+      f1    int
+      b1    B
+    }
+
+    struct B {
+      s1    string
+    }
+
+    struct C extends common.B {
+      b1    bool
+    }
+
+    struct D {
+       a1    common.A
+       c1    C
+       dir   SortDir
+    }
+
+    enum SortDir {
+        ASC
+        DESC
+    }
+    """
+      expected = [ { "name" : "common.A", "type" : "struct", 
+                       "extends" : "", "comment" : "",
+                       "fields" : [ field("f1", "int"), 
+                                    field("b1", "common.B") ] },
+                    { "name" : "common.B", "type" : "struct", 
+                       "extends" : "", "comment" : "",
+                       "fields" : [ field("s1", "string") ] },
+                    { "name" : "common.C", "type" : "struct", 
+                       "extends" : "common.B", "comment" : "",
+                       "fields" : [ field("b1", "bool") ] },
+                    { "name" : "common.D", "type" : "struct", 
+                       "extends" : "", "comment" : "",
+                       "fields" : [ field("a1", "common.A"), 
+                                    field("c1", "common.C"),
+                                    field("dir", "common.SortDir") ] },
+                    { "name" : "common.SortDir", "type" : "enum", 
+                      "comment" : "",
+                      "values" : [ 
+                          { "comment" : "", "value": "ASC" },
+                          { "comment" : "", "value": "DESC" }
+                      ] }
+      ]
+      self.assertEquals(expected, parse(idl, validate=False, add_meta=False))
+
+    def test_cannot_namespace_interfaces(self):
+        idl = """namespace foo
+
+interface FooService {
+  doStuff() bool
+}
+        """
+        try:
+            parse(idl, add_meta=False)
+            self.fail("should have thrown exception")
+        except IdlParseException as e:
+            expected = [ { "line": 3, "message" : "namespace cannot be used in files with interfaces" } ]
+            self.assertEquals(expected, e.errors)
+
+    def test_cannot_redefine_namespace(self):
+        idl = """namespace foo
+namespace bar
+        """
+        try:
+            parse(idl, add_meta=False)
+            self.fail("should have thrown exception")
+        except IdlParseException as e:
+            expected = [ { "line": 2, "message" : "Cannot redeclare namespace" } ]
+            self.assertEquals(expected, e.errors)
+
+    def test_namespace_must_preceed_elems(self):
+        idl = """enum Status { 
+        A
+        B
+        }
+
+namespace foo
+
+struct Blah { 
+    name string
+}
+        """
+        try:
+            parse(idl, add_meta=False)
+            self.fail("should have thrown exception")
+        except IdlParseException as e:
+            expected = [ { "line": 6, "message" : "namespace must preceed all struct/enum/interface definitions" } ]
+            self.assertEquals(expected, e.errors)  
+
     def test_add_meta(self):
         idl = """interface FooService {
     do_foo() string
@@ -462,5 +561,30 @@ interface FooService {
             meta = parsed[-1]
             self.assertNotEquals(first_checksum, meta["checksum"])
 
+    def test_file_paths(self):
+      # [0] = relative filename
+      # [1] = search path dirs
+      # [2] = expected paths returned
+
+      # set BARRISTER_TEST, which should be used if no search path is provided
+      os.environ["BARRISTER_PATH"] = "/dir1:/dir2"
+      tests = [
+        [ "a.idl", "/tmp:/usr/local", [ "a.idl", "/tmp/a.idl", "/usr/local/a.idl"] ],
+        [ "../b.idl", None, [ "../b.idl", "/dir1/../b.idl", "/dir2/../b.idl" ] ]
+      ]
+      for test in tests:
+        paths = file_paths(test[0], test[1])
+        self.assertEquals(test[2], paths)
+
+      # remove BARRISTER_PATH and re-test
+      del os.environ["BARRISTER_PATH"]
+      tests = [
+        [ "a.idl", "/tmp:/usr/local", [ "a.idl", "/tmp/a.idl", "/usr/local/a.idl"] ],
+        [ "../b.idl", None, [ "../b.idl" ] ]
+      ]
+      for test in tests:
+        paths = file_paths(test[0], test[1])
+        self.assertEquals(test[2], paths)
+
 if __name__ == "__main__":
     unittest.main()
